# Copyright (C) 2010 Richard Lincoln
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

""" This package is an extension of the Metering package and contains the information classes that support specialized applications such as demand-side management using load control equipment. These classes are generally associated with the point where a service is delivered to the customer.
"""

from cim.iec61968.metering import DeviceFunction
from cim import Element

# <<< imports
# @generated
# >>> imports

ns_prefix = "cim.loadcontrol"

ns_uri = "http://iec.ch/TC57/2009/CIM-schema-cim14#LoadControl"

class ConnectDisconnectFunction(DeviceFunction):
    """ A function that will disconnect or reconnect the customer's load under defined conditions.
    """
    # <<< connect_disconnect_function
    # @generated
    def __init__(self, is_delayed_discon=False, is_connected=False, event_count=0, is_local_auto_discon_op=False, is_remote_auto_discon_op=False, is_remote_auto_recon_op=False, is_local_auto_recon_op=False, rcd_info=None, switches=None, **kw_args):
        """ Initialises a new 'ConnectDisconnectFunction' instance.
        """
        # If set true, the switch may disconnect the service at the end of a specified time delay after the disconnect signal had been given. If set false, the switch may disconnect the service immediately after the disconnect signal had been given. This is typically the case for over current circuit-breakers which are classified as either instantaneous or slow acting. 
        self.is_delayed_discon = is_delayed_discon

        # True if this function is in the connected state. 
        self.is_connected = is_connected

        # Running cumulative count of (connect or disconnect) events, for the lifetime of this function or until the value is cleared. 
        self.event_count = event_count

        # (if disconnection can be operated locally) If set true, the operation happens automatically, otherwise it happens manually. 
        self.is_local_auto_discon_op = is_local_auto_discon_op

        # If set true and if disconnection can be operated remotely, then the operation happens automatically. If set false and if disconnection can be operated remotely, then the operation happens manually. 
        self.is_remote_auto_discon_op = is_remote_auto_discon_op

        # If set true and if reconnection can be operated remotely, then the operation happens automatically. If false and if reconnection can be operated remotely, then the operation happens manually. 
        self.is_remote_auto_recon_op = is_remote_auto_recon_op

        # If set true and if reconnection can be operated locally, then the operation happens automatically. Otherwise, it is manually. 
        self.is_local_auto_recon_op = is_local_auto_recon_op


        self.rcd_info = rcd_info

        self._switches = []
        if switches is not None:
            self.switches = switches
        else:
            self.switches = []


        super(ConnectDisconnectFunction, self).__init__(**kw_args)
    # >>> connect_disconnect_function

    # <<< rcd_info
    # @generated
    # Information on remote connect disconnect switch.
    rcd_info = None
    # >>> rcd_info

    # <<< switches
    # @generated
    def get_switches(self):
        """ 
        """
        return self._switches

    def set_switches(self, value):
        for p in self._switches:
            filtered = [q for q in p.connect_disconnect_functions if q != self]
            self._switches._connect_disconnect_functions = filtered
        for r in value:
            if self not in r._connect_disconnect_functions:
                r._connect_disconnect_functions.append(self)
        self._switches = value

    switches = property(get_switches, set_switches)

    def add_switches(self, *switches):
        for obj in switches:
            if self not in obj._connect_disconnect_functions:
                obj._connect_disconnect_functions.append(self)
            self._switches.append(obj)

    def remove_switches(self, *switches):
        for obj in switches:
            if self in obj._connect_disconnect_functions:
                obj._connect_disconnect_functions.remove(self)
            self._switches.remove(obj)
    # >>> switches


    def __str__(self):
        """ Returns a string representation of the ConnectDisconnectFunction.
        """
        return self.serialize(header=True, depth=2, format=True)


    # <<< connect_disconnect_function.serialize
    # @generated
    def serialize(self, header=False, depth=0, format=False):
        """ Returns an RDF/XML representation of the ConnectDisconnectFunction.
        """
        s = ''
        indent = ' ' * depth if depth else ''
        if format:
            indent = '\n' + indent
        if header:
            s += '<?xml version="1.0" encoding="UTF-8"?>\n'
            s += '<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:%s="%s">' % \
                (ns_prefix, ns_uri)
            if format:
                indent += ' ' * depth

        s += '%s<%s:%s rdf:ID="%s">' % (indent, ns_prefix, "ConnectDisconnectFunction", self.uri)
        if format:
            indent += ' ' * depth

        if self.rcd_info is not None:
            s += '%s<%s:ConnectDisconnectFunction.rcd_info rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.rcd_info.uri)
        for obj in self.switches:
            s += '%s<%s:ConnectDisconnectFunction.switches rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        s += '%s<%s:ConnectDisconnectFunction.is_delayed_discon>%s</%s:ConnectDisconnectFunction.is_delayed_discon>' % \
            (indent, ns_prefix, self.is_delayed_discon, ns_prefix)
        s += '%s<%s:ConnectDisconnectFunction.is_connected>%s</%s:ConnectDisconnectFunction.is_connected>' % \
            (indent, ns_prefix, self.is_connected, ns_prefix)
        s += '%s<%s:ConnectDisconnectFunction.event_count>%s</%s:ConnectDisconnectFunction.event_count>' % \
            (indent, ns_prefix, self.event_count, ns_prefix)
        s += '%s<%s:ConnectDisconnectFunction.is_local_auto_discon_op>%s</%s:ConnectDisconnectFunction.is_local_auto_discon_op>' % \
            (indent, ns_prefix, self.is_local_auto_discon_op, ns_prefix)
        s += '%s<%s:ConnectDisconnectFunction.is_remote_auto_discon_op>%s</%s:ConnectDisconnectFunction.is_remote_auto_discon_op>' % \
            (indent, ns_prefix, self.is_remote_auto_discon_op, ns_prefix)
        s += '%s<%s:ConnectDisconnectFunction.is_remote_auto_recon_op>%s</%s:ConnectDisconnectFunction.is_remote_auto_recon_op>' % \
            (indent, ns_prefix, self.is_remote_auto_recon_op, ns_prefix)
        s += '%s<%s:ConnectDisconnectFunction.is_local_auto_recon_op>%s</%s:ConnectDisconnectFunction.is_local_auto_recon_op>' % \
            (indent, ns_prefix, self.is_local_auto_recon_op, ns_prefix)
        if self.parent is not None:
            s += '%s<%s:Element.parent rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.parent.uri)
        s += '%s<%s:Element.uuid>%s</%s:Element.uuid>' % \
            (indent, ns_prefix, self.uuid, ns_prefix)
        if self.modeling_authority_set is not None:
            s += '%s<%s:IdentifiedObject.modeling_authority_set rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.modeling_authority_set.uri)
        s += '%s<%s:IdentifiedObject.description>%s</%s:IdentifiedObject.description>' % \
            (indent, ns_prefix, self.description, ns_prefix)
        s += '%s<%s:IdentifiedObject.m_rid>%s</%s:IdentifiedObject.m_rid>' % \
            (indent, ns_prefix, self.m_rid, ns_prefix)
        s += '%s<%s:IdentifiedObject.name>%s</%s:IdentifiedObject.name>' % \
            (indent, ns_prefix, self.name, ns_prefix)
        s += '%s<%s:IdentifiedObject.path_name>%s</%s:IdentifiedObject.path_name>' % \
            (indent, ns_prefix, self.path_name, ns_prefix)
        s += '%s<%s:IdentifiedObject.local_name>%s</%s:IdentifiedObject.local_name>' % \
            (indent, ns_prefix, self.local_name, ns_prefix)
        s += '%s<%s:IdentifiedObject.alias_name>%s</%s:IdentifiedObject.alias_name>' % \
            (indent, ns_prefix, self.alias_name, ns_prefix)
        for obj in self.measurements:
            s += '%s<%s:Asset.measurements rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.hazards:
            s += '%s<%s:Asset.hazards rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.erp_organisation_roles:
            s += '%s<%s:Asset.erp_organisation_roles rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        if self.dimensions_info is not None:
            s += '%s<%s:Asset.dimensions_info rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.dimensions_info.uri)
        for obj in self.scheduled_events:
            s += '%s<%s:Asset.scheduled_events rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.mediums:
            s += '%s<%s:Asset.mediums rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.asset_functions:
            s += '%s<%s:Asset.asset_functions rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.properties:
            s += '%s<%s:Asset.properties rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        if self.asset_container is not None:
            s += '%s<%s:Asset.asset_container rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.asset_container.uri)
        for obj in self.ratings:
            s += '%s<%s:Asset.ratings rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.activity_records:
            s += '%s<%s:Asset.activity_records rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.from_asset_roles:
            s += '%s<%s:Asset.from_asset_roles rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.location_roles:
            s += '%s<%s:Asset.location_roles rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.power_system_resource_roles:
            s += '%s<%s:Asset.power_system_resource_roles rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.document_roles:
            s += '%s<%s:Asset.document_roles rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.change_items:
            s += '%s<%s:Asset.change_items rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        if self.erp_item_master is not None:
            s += '%s<%s:Asset.erp_item_master rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.erp_item_master.uri)
        for obj in self.electronic_addresses:
            s += '%s<%s:Asset.electronic_addresses rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        if self.work_task is not None:
            s += '%s<%s:Asset.work_task rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.work_task.uri)
        for obj in self.erp_rec_delivery_items:
            s += '%s<%s:Asset.erp_rec_delivery_items rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.reliability_infos:
            s += '%s<%s:Asset.reliability_infos rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.to_asset_roles:
            s += '%s<%s:Asset.to_asset_roles rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.asset_property_curves:
            s += '%s<%s:Asset.asset_property_curves rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        if self.financial_info is not None:
            s += '%s<%s:Asset.financial_info rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.financial_info.uri)
        if self.erp_inventory is not None:
            s += '%s<%s:Asset.erp_inventory rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.erp_inventory.uri)
        if self.acceptance_test is not None:
            s += '%s<%s:Asset.acceptance_test rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.acceptance_test.uri)
        if self.status is not None:
            s += '%s<%s:Asset.status rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.status.uri)
        s += '%s<%s:Asset.initial_condition>%s</%s:Asset.initial_condition>' % \
            (indent, ns_prefix, self.initial_condition, ns_prefix)
        s += '%s<%s:Asset.category>%s</%s:Asset.category>' % \
            (indent, ns_prefix, self.category, ns_prefix)
        s += '%s<%s:Asset.lot_number>%s</%s:Asset.lot_number>' % \
            (indent, ns_prefix, self.lot_number, ns_prefix)
        s += '%s<%s:Asset.application>%s</%s:Asset.application>' % \
            (indent, ns_prefix, self.application, ns_prefix)
        s += '%s<%s:Asset.serial_number>%s</%s:Asset.serial_number>' % \
            (indent, ns_prefix, self.serial_number, ns_prefix)
        s += '%s<%s:Asset.installation_date>%s</%s:Asset.installation_date>' % \
            (indent, ns_prefix, self.installation_date, ns_prefix)
        s += '%s<%s:Asset.corporate_code>%s</%s:Asset.corporate_code>' % \
            (indent, ns_prefix, self.corporate_code, ns_prefix)
        s += '%s<%s:Asset.purchase_price>%s</%s:Asset.purchase_price>' % \
            (indent, ns_prefix, self.purchase_price, ns_prefix)
        s += '%s<%s:Asset.manufactured_date>%s</%s:Asset.manufactured_date>' % \
            (indent, ns_prefix, self.manufactured_date, ns_prefix)
        s += '%s<%s:Asset.initial_loss_of_life>%s</%s:Asset.initial_loss_of_life>' % \
            (indent, ns_prefix, self.initial_loss_of_life, ns_prefix)
        s += '%s<%s:Asset.utc_number>%s</%s:Asset.utc_number>' % \
            (indent, ns_prefix, self.utc_number, ns_prefix)
        s += '%s<%s:Asset.critical>%s</%s:Asset.critical>' % \
            (indent, ns_prefix, self.critical, ns_prefix)
        if self.asset is not None:
            s += '%s<%s:AssetFunction.asset rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.asset.uri)
        if self.asset_function_asset_model is not None:
            s += '%s<%s:AssetFunction.asset_function_asset_model rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.asset_function_asset_model.uri)
        s += '%s<%s:AssetFunction.hardware_id>%s</%s:AssetFunction.hardware_id>' % \
            (indent, ns_prefix, self.hardware_id, ns_prefix)
        s += '%s<%s:AssetFunction.config_id>%s</%s:AssetFunction.config_id>' % \
            (indent, ns_prefix, self.config_id, ns_prefix)
        s += '%s<%s:AssetFunction.program_id>%s</%s:AssetFunction.program_id>' % \
            (indent, ns_prefix, self.program_id, ns_prefix)
        s += '%s<%s:AssetFunction.password>%s</%s:AssetFunction.password>' % \
            (indent, ns_prefix, self.password, ns_prefix)
        s += '%s<%s:AssetFunction.firmware_id>%s</%s:AssetFunction.firmware_id>' % \
            (indent, ns_prefix, self.firmware_id, ns_prefix)
        if self.com_equipment_asset is not None:
            s += '%s<%s:DeviceFunction.com_equipment_asset rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.com_equipment_asset.uri)
        if self.end_device_asset is not None:
            s += '%s<%s:DeviceFunction.end_device_asset rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.end_device_asset.uri)
        for obj in self.registers:
            s += '%s<%s:DeviceFunction.registers rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        for obj in self.end_device_events:
            s += '%s<%s:DeviceFunction.end_device_events rdf:resource="#%s"/>' % \
                (indent, ns_prefix, obj.uri)
        s += '%s<%s:DeviceFunction.disabled>%s</%s:DeviceFunction.disabled>' % \
            (indent, ns_prefix, self.disabled, ns_prefix)

        if format:
            indent = indent[:-depth]
        s += '%s</%s:%s>' % (indent, ns_prefix, "ConnectDisconnectFunction")

        if header:
            s += '%s</rdf:RDF>' % indent[:-depth]

        return s
    # >>> connect_disconnect_function.serialize


class RemoteConnectDisconnectInfo(Element):
    """ Details of remote connect disconnect function.
    """
    # <<< remote_connect_disconnect_info
    # @generated
    def __init__(self, is_arm_connect=False, customer_voltage_limit=0.0, power_limit=0.0, is_energy_limiting=False, energy_usage_warning=0.0, is_arm_disconnect=False, energy_usage_start_date_time='', needs_voltage_limit_check=False, energy_limit=0.0, armed_timeout=0.0, needs_power_limit_check=False, use_pushbutton=False, **kw_args):
        """ Initialises a new 'RemoteConnectDisconnectInfo' instance.
        """
        # True if the RCD switch must be armed before a connect action can be initiated. 
        self.is_arm_connect = is_arm_connect

        # Voltage limit on customer side of RCD switch above which the connect should not be made. 
        self.customer_voltage_limit = customer_voltage_limit

        # Load limit above which the connect should either not take place or should cause an immediate disconnect. 
        self.power_limit = power_limit

        # True if the energy usage is limited and the customer will be disconnected if they go over the limit. 
        self.is_energy_limiting = is_energy_limiting

        # Warning energy limit, used to trigger event code that energy usage is nearing limit. 
        self.energy_usage_warning = energy_usage_warning

        # True if the RCD switch must be armed before a disconnect action can be initiated. 
        self.is_arm_disconnect = is_arm_disconnect

        # Start date and time to accumulate energy for energy usage limiting. 
        self.energy_usage_start_date_time = energy_usage_start_date_time

        # True if voltage limit must be checked to prevent connect if voltage is over the limit. 
        self.needs_voltage_limit_check = needs_voltage_limit_check

        # Limit of energy before disconnect. 
        self.energy_limit = energy_limit

        # Setting of the timeout elapsed time. 
        self.armed_timeout = armed_timeout

        # True if load limit must be checked to issue an immediate disconnect (after a connect) if load is over the limit. 
        self.needs_power_limit_check = needs_power_limit_check

        # True if pushbutton must be used for connect. 
        self.use_pushbutton = use_pushbutton



        super(RemoteConnectDisconnectInfo, self).__init__(**kw_args)
    # >>> remote_connect_disconnect_info


    def __str__(self):
        """ Returns a string representation of the RemoteConnectDisconnectInfo.
        """
        return self.serialize(header=True, depth=2, format=True)


    # <<< remote_connect_disconnect_info.serialize
    # @generated
    def serialize(self, header=False, depth=0, format=False):
        """ Returns an RDF/XML representation of the RemoteConnectDisconnectInfo.
        """
        s = ''
        indent = ' ' * depth if depth else ''
        if format:
            indent = '\n' + indent
        if header:
            s += '<?xml version="1.0" encoding="UTF-8"?>\n'
            s += '<rdf:RDF xmlns:rdf="http://www.w3.org/1999/02/22-rdf-syntax-ns#" xmlns:%s="%s">' % \
                (ns_prefix, ns_uri)
            if format:
                indent += ' ' * depth

        s += '%s<%s:%s rdf:ID="%s">' % (indent, ns_prefix, "RemoteConnectDisconnectInfo", self.uri)
        if format:
            indent += ' ' * depth

        s += '%s<%s:RemoteConnectDisconnectInfo.is_arm_connect>%s</%s:RemoteConnectDisconnectInfo.is_arm_connect>' % \
            (indent, ns_prefix, self.is_arm_connect, ns_prefix)
        s += '%s<%s:RemoteConnectDisconnectInfo.customer_voltage_limit>%s</%s:RemoteConnectDisconnectInfo.customer_voltage_limit>' % \
            (indent, ns_prefix, self.customer_voltage_limit, ns_prefix)
        s += '%s<%s:RemoteConnectDisconnectInfo.power_limit>%s</%s:RemoteConnectDisconnectInfo.power_limit>' % \
            (indent, ns_prefix, self.power_limit, ns_prefix)
        s += '%s<%s:RemoteConnectDisconnectInfo.is_energy_limiting>%s</%s:RemoteConnectDisconnectInfo.is_energy_limiting>' % \
            (indent, ns_prefix, self.is_energy_limiting, ns_prefix)
        s += '%s<%s:RemoteConnectDisconnectInfo.energy_usage_warning>%s</%s:RemoteConnectDisconnectInfo.energy_usage_warning>' % \
            (indent, ns_prefix, self.energy_usage_warning, ns_prefix)
        s += '%s<%s:RemoteConnectDisconnectInfo.is_arm_disconnect>%s</%s:RemoteConnectDisconnectInfo.is_arm_disconnect>' % \
            (indent, ns_prefix, self.is_arm_disconnect, ns_prefix)
        s += '%s<%s:RemoteConnectDisconnectInfo.energy_usage_start_date_time>%s</%s:RemoteConnectDisconnectInfo.energy_usage_start_date_time>' % \
            (indent, ns_prefix, self.energy_usage_start_date_time, ns_prefix)
        s += '%s<%s:RemoteConnectDisconnectInfo.needs_voltage_limit_check>%s</%s:RemoteConnectDisconnectInfo.needs_voltage_limit_check>' % \
            (indent, ns_prefix, self.needs_voltage_limit_check, ns_prefix)
        s += '%s<%s:RemoteConnectDisconnectInfo.energy_limit>%s</%s:RemoteConnectDisconnectInfo.energy_limit>' % \
            (indent, ns_prefix, self.energy_limit, ns_prefix)
        s += '%s<%s:RemoteConnectDisconnectInfo.armed_timeout>%s</%s:RemoteConnectDisconnectInfo.armed_timeout>' % \
            (indent, ns_prefix, self.armed_timeout, ns_prefix)
        s += '%s<%s:RemoteConnectDisconnectInfo.needs_power_limit_check>%s</%s:RemoteConnectDisconnectInfo.needs_power_limit_check>' % \
            (indent, ns_prefix, self.needs_power_limit_check, ns_prefix)
        s += '%s<%s:RemoteConnectDisconnectInfo.use_pushbutton>%s</%s:RemoteConnectDisconnectInfo.use_pushbutton>' % \
            (indent, ns_prefix, self.use_pushbutton, ns_prefix)
        if self.parent is not None:
            s += '%s<%s:Element.parent rdf:resource="#%s"/>' % \
                (indent, ns_prefix, self.parent.uri)
        s += '%s<%s:Element.uuid>%s</%s:Element.uuid>' % \
            (indent, ns_prefix, self.uuid, ns_prefix)

        if format:
            indent = indent[:-depth]
        s += '%s</%s:%s>' % (indent, ns_prefix, "RemoteConnectDisconnectInfo")

        if header:
            s += '%s</rdf:RDF>' % indent[:-depth]

        return s
    # >>> remote_connect_disconnect_info.serialize


# <<< load_control
# @generated
# >>> load_control



#------------------------------------------------------------------------------
# Copyright (C) 2010 Richard Lincoln
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#------------------------------------------------------------------------------

""" The OperationalLimits package models a specification of limits associated with equipment and other operational entities.
"""
#------------------------------------------------------------------------------
#  Imports:
#------------------------------------------------------------------------------

from CIM.IEC61970.Core import IdentifiedObject
from CIM import Element
from CIM.IEC61970.Domain import Voltage
from CIM.IEC61970.Domain import ApparentPower
from CIM.IEC61970.Domain import ActivePower
from CIM.IEC61970.Domain import ReactivePower
from CIM.IEC61970.Domain import CurrentFlow
from CIM.IEC61970.Domain import Seconds



from enthought.traits.api import Instance, List, Property, Enum, Str, Bool
# <<< imports
# @generated
from enthought.traits.ui.api import View, Group, Item, HGroup, VGroup, Tabbed, VGrid, InstanceEditor
# >>> imports
#------------------------------------------------------------------------------
#  Trait definitions:
#------------------------------------------------------------------------------

# The direction of an operational limit.
OperationalLimitDirectionKind = Enum("absoluteValue", "high", "low", desc="The direction of an operational limit.")

#------------------------------------------------------------------------------
#  "OperationalLimit" class:
#------------------------------------------------------------------------------

class OperationalLimit(IdentifiedObject):
    """ A value associated with a specific kind of limit.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # The limit type associated with this limit.
    OperationalLimitType = Instance("CIM.IEC61970.OperationalLimits.OperationalLimitType",
        desc="The limit type associated with this limit.",
        transient=True,
        opposite="OperationalLimit",
        editor=InstanceEditor(name="_operationallimittypes"))

    def _get_operationallimittypes(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61970.OperationalLimits.OperationalLimitType" ]
        else:
            return []

    _operationallimittypes = Property(fget=_get_operationallimittypes)

    # The limit set to which the limit values belong.
    OperationalLimitSet = Instance("CIM.IEC61970.OperationalLimits.OperationalLimitSet",
        desc="The limit set to which the limit values belong.",
        transient=True,
        opposite="OperationalLimitValue",
        editor=InstanceEditor(name="_operationallimitsets"))

    def _get_operationallimitsets(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61970.OperationalLimits.OperationalLimitSet" ]
        else:
            return []

    _operationallimitsets = Property(fget=_get_operationallimitsets)

    # Used to specify high/low and limit levels.
    type = Str(desc="Used to specify high/low and limit levels.")

    #--------------------------------------------------------------------------
    #  Begin "OperationalLimit" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "type",
                label="Attributes"),
            VGroup("Parent", "ModelingAuthoritySet", "OperationalLimitType", "OperationalLimitSet",
                label="References"),
            dock="tab"),
        id="CIM.IEC61970.OperationalLimits.OperationalLimit",
        title="OperationalLimit",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "OperationalLimit" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "BranchGroup" class:
#------------------------------------------------------------------------------

class BranchGroup(IdentifiedObject):
    """ A group of branch terminals whose directed flow summation is to be monitored. Abranch group need not form a cutset of the network.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # The directed branch group terminals to be summed.
    BranchGroupTerminal = List(Instance("CIM.IEC61970.OperationalLimits.BranchGroupTerminal"),
        desc="The directed branch group terminals to be summed.")

    # The maximum active power flow.
    maximumActivePower = ActivePower(desc="The maximum active power flow.")

    # Monitor the reactive power flow.
    monitorReactivePower = Bool(desc="Monitor the reactive power flow.")

    # The minimum active power flow.
    minimumActivePower = ActivePower(desc="The minimum active power flow.")

    # The minimum reactive power flow.
    minimumReactivePower = ReactivePower(desc="The minimum reactive power flow.")

    # Monitor the active power flow.
    monitorActivePower = Bool(desc="Monitor the active power flow.")

    # The maximum reactive power flow.
    maximumReactivePower = ReactivePower(desc="The maximum reactive power flow.")

    #--------------------------------------------------------------------------
    #  Begin "BranchGroup" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "maximumActivePower", "monitorReactivePower", "minimumActivePower", "minimumReactivePower", "monitorActivePower", "maximumReactivePower",
                label="Attributes", columns=1),
            VGroup("Parent", "ModelingAuthoritySet", "BranchGroupTerminal",
                label="References"),
            dock="tab"),
        id="CIM.IEC61970.OperationalLimits.BranchGroup",
        title="BranchGroup",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "BranchGroup" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "BranchGroupTerminal" class:
#------------------------------------------------------------------------------

class BranchGroupTerminal(Element):
    """ A specific directed terminal flow for a branch group.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # The branch group to which the directed branch group terminals belong.
    BranchGroup = Instance("CIM.IEC61970.OperationalLimits.BranchGroup",
        desc="The branch group to which the directed branch group terminals belong.",
        transient=True,
        opposite="BranchGroupTerminal",
        editor=InstanceEditor(name="_branchgroups"))

    def _get_branchgroups(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61970.OperationalLimits.BranchGroup" ]
        else:
            return []

    _branchgroups = Property(fget=_get_branchgroups)

    # The terminal to be summed.
    Terminal = Instance("CIM.IEC61970.Core.Terminal",
        desc="The terminal to be summed.",
        transient=True,
        opposite="BranchGroupTerminal",
        editor=InstanceEditor(name="_terminals"))

    def _get_terminals(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61970.Core.Terminal" ]
        else:
            return []

    _terminals = Property(fget=_get_terminals)

    # The flow into the terminal is summed if set true.   The flow out of the terminanl is summed if set false.
    positiveFlowIn = Bool(desc="The flow into the terminal is summed if set true.   The flow out of the terminanl is summed if set false.")

    #--------------------------------------------------------------------------
    #  Begin "BranchGroupTerminal" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "positiveFlowIn",
                label="Attributes"),
            VGroup("Parent", "BranchGroup", "Terminal",
                label="References"),
            dock="tab"),
        id="CIM.IEC61970.OperationalLimits.BranchGroupTerminal",
        title="BranchGroupTerminal",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "BranchGroupTerminal" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "OperationalLimitSet" class:
#------------------------------------------------------------------------------

class OperationalLimitSet(IdentifiedObject):
    """ A set of limits associated with equipmnet.  Sets of limits might apply to a specific temperature, or season for example. A set of limits may contain may different severities of limit levels that would apply to the same equipment.   The set may contain limits of different types such as apparent power and current limits or high and low voltage limits  that are logically applied together as a set.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # The terminal specifically associated to this operational limit set.  If no terminal is associated, all terminals of the equipment are implied.
    Terminal = Instance("CIM.IEC61970.Core.Terminal",
        desc="The terminal specifically associated to this operational limit set.  If no terminal is associated, all terminals of the equipment are implied.",
        transient=True,
        opposite="OperationalLimitSet",
        editor=InstanceEditor(name="_terminals"))

    def _get_terminals(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61970.Core.Terminal" ]
        else:
            return []

    _terminals = Property(fget=_get_terminals)

    # The equpment to which the limit set applies.
    Equipment = Instance("CIM.IEC61970.Core.Equipment",
        desc="The equpment to which the limit set applies.",
        transient=True,
        opposite="OperationalLimitSet",
        editor=InstanceEditor(name="_equipments"))

    def _get_equipments(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61970.Core.Equipment" ]
        else:
            return []

    _equipments = Property(fget=_get_equipments)

    # Values of equipment limits.
    OperationalLimitValue = List(Instance("CIM.IEC61970.OperationalLimits.OperationalLimit"),
        desc="Values of equipment limits.")

    #--------------------------------------------------------------------------
    #  Begin "OperationalLimitSet" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName",
                label="Attributes"),
            VGroup("Parent", "ModelingAuthoritySet", "Terminal", "Equipment", "OperationalLimitValue",
                label="References"),
            dock="tab"),
        id="CIM.IEC61970.OperationalLimits.OperationalLimitSet",
        title="OperationalLimitSet",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "OperationalLimitSet" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "OperationalLimitType" class:
#------------------------------------------------------------------------------

class OperationalLimitType(Element):
    """ A type of limit.  The meaning of a specific limit is described in this class.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # The operational limits associated with this type of limit.
    OperationalLimit = List(Instance("CIM.IEC61970.OperationalLimits.OperationalLimit"),
        desc="The operational limits associated with this type of limit.")

    # The direction of the limit.
    direction = OperationalLimitDirectionKind(desc="The direction of the limit.")

    # The nominal acceptable duration of the limit.  Limits are commonly expressed in terms of the a time limit for which the limit is normally acceptable.   The actual acceptable duration of a specific limit may depend on other local factors such as temperature or wind speed.
    acceptableDuration = Seconds(desc="The nominal acceptable duration of the limit.  Limits are commonly expressed in terms of the a time limit for which the limit is normally acceptable.   The actual acceptable duration of a specific limit may depend on other local factors such as temperature or wind speed.")

    #--------------------------------------------------------------------------
    #  Begin "OperationalLimitType" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "direction", "acceptableDuration",
                label="Attributes"),
            VGroup("Parent", "OperationalLimit",
                label="References"),
            dock="tab"),
        id="CIM.IEC61970.OperationalLimits.OperationalLimitType",
        title="OperationalLimitType",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "OperationalLimitType" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "VoltageLimit" class:
#------------------------------------------------------------------------------

class VoltageLimit(OperationalLimit):
    """ Operational limit applied to voltage.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # Limit on voltage. High or low limit depends on the OperatoinalLimit.limitKind
    value = Voltage(desc="Limit on voltage. High or low limit depends on the OperatoinalLimit.limitKind")

    #--------------------------------------------------------------------------
    #  Begin "VoltageLimit" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "type", "value",
                label="Attributes"),
            VGroup("Parent", "ModelingAuthoritySet", "OperationalLimitType", "OperationalLimitSet",
                label="References"),
            dock="tab"),
        id="CIM.IEC61970.OperationalLimits.VoltageLimit",
        title="VoltageLimit",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "VoltageLimit" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "ApparentPowerLimit" class:
#------------------------------------------------------------------------------

class ApparentPowerLimit(OperationalLimit):
    """ Apparent power limit.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # The apparent power limit.
    value = ApparentPower(desc="The apparent power limit.")

    #--------------------------------------------------------------------------
    #  Begin "ApparentPowerLimit" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "type", "value",
                label="Attributes"),
            VGroup("Parent", "ModelingAuthoritySet", "OperationalLimitType", "OperationalLimitSet",
                label="References"),
            dock="tab"),
        id="CIM.IEC61970.OperationalLimits.ApparentPowerLimit",
        title="ApparentPowerLimit",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "ApparentPowerLimit" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "ActivePowerLimit" class:
#------------------------------------------------------------------------------

class ActivePowerLimit(OperationalLimit):
    """ Limit on active power flow.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # Value of active power limit.
    value = ActivePower(desc="Value of active power limit.")

    #--------------------------------------------------------------------------
    #  Begin "ActivePowerLimit" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "type", "value",
                label="Attributes"),
            VGroup("Parent", "ModelingAuthoritySet", "OperationalLimitType", "OperationalLimitSet",
                label="References"),
            dock="tab"),
        id="CIM.IEC61970.OperationalLimits.ActivePowerLimit",
        title="ActivePowerLimit",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "ActivePowerLimit" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "CurrentLimit" class:
#------------------------------------------------------------------------------

class CurrentLimit(OperationalLimit):
    """ Operational limit on current.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # Limit on current flow.
    value = CurrentFlow(desc="Limit on current flow.")

    #--------------------------------------------------------------------------
    #  Begin "CurrentLimit" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "type", "value",
                label="Attributes"),
            VGroup("Parent", "ModelingAuthoritySet", "OperationalLimitType", "OperationalLimitSet",
                label="References"),
            dock="tab"),
        id="CIM.IEC61970.OperationalLimits.CurrentLimit",
        title="CurrentLimit",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "CurrentLimit" user definitions:
    #--------------------------------------------------------------------------



# EOF -------------------------------------------------------------------------



#------------------------------------------------------------------------------
# Copyright (C) 2010 Richard Lincoln
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#------------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  Imports:
#------------------------------------------------------------------------------

from CIM.IEC61968.Metering import DeviceFunction
from CIM.IEC61968.Common import ActivityRecord
from CIM.IEC61970.Domain import Minutes
from CIM.IEC61970.Domain import Seconds
from CIM.IEC61970.Domain import ActivePower



from enthought.traits.api import Instance, List, Property, Enum, Bool
# <<< imports
# @generated
from enthought.traits.ui.api import View, Group, Item, HGroup, VGroup, Tabbed, VGrid, InstanceEditor
# >>> imports
#------------------------------------------------------------------------------
#  Trait definitions:
#------------------------------------------------------------------------------

# Scheduling basis for a load management function.
LoadMgmtKind = Enum("tariffBased", "remoteControl", "manualControl", "timeBased", desc="Scheduling basis for a load management function.")
# Different states of the load.
LoadStateKind = Enum("noLoad", "fullLoad", "limitedLoad", desc="Different states of the load.")

#------------------------------------------------------------------------------
#  "LoadMgmtFunction" class:
#------------------------------------------------------------------------------

class LoadMgmtFunction(DeviceFunction):
    """ A collective function at an end device that manages the customer load.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    Switches = List(Instance("CIM.IEC61970.Wires.Switch"))

    LoadMgmtRecords = List(Instance("CIM.IEC61968.Informative.InfLoadControl.LoadMgmtRecord"))

    # The basis of Load Management scheduling used here: Time Based, Tariff Based, Remote Control and Manual Control.
    schedulingBasis = LoadMgmtKind(desc="The basis of Load Management scheduling used here: Time Based, Tariff Based, Remote Control and Manual Control.")

    # The present state of the load being either shed (noLoad), limited (limitedLoad) or fully connected (fullLoad). This refers only to the portion of the customer load that is under control of the LoadMgmtFunction.
    loadStatus = LoadStateKind(desc="The present state of the load being either shed (noLoad), limited (limitedLoad) or fully connected (fullLoad). This refers only to the portion of the customer load that is under control of the LoadMgmtFunction.")

    # True if the currently active schedule is being manually over-ridden to either shed load or to limit load.
    manualOverRide = Bool(desc="True if the currently active schedule is being manually over-ridden to either shed load or to limit load.")

    # After a command had been received to activate the mannualOverRide state or remoteOverRideState, the normal (halted) schedule will resume after this specified time duration had elapsed.
    overRideTimeOut = Minutes(desc="After a command had been received to activate the mannualOverRide state or remoteOverRideState, the normal (halted) schedule will resume after this specified time duration had elapsed.")

    # True if the currently active schedule is being remotely over-ridden to either shed load or to limit load.
    remoteOverRide = Bool(desc="True if the currently active schedule is being remotely over-ridden to either shed load or to limit load.")

    # True if LoadMgmtFunction operates under automatic control, otherwise it operates under manual control.
    isAutoOp = Bool(desc="True if LoadMgmtFunction operates under automatic control, otherwise it operates under manual control.")

    #--------------------------------------------------------------------------
    #  Begin "LoadMgmtFunction" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "initialCondition", "category", "lotNumber", "application", "serialNumber", "installationDate", "corporateCode", "purchasePrice", "manufacturedDate", "initialLossOfLife", "utcNumber", "critical", "hardwareID", "configID", "programID", "password", "firmwareID", "disabled", "schedulingBasis", "loadStatus", "manualOverRide", "overRideTimeOut", "remoteOverRide", "isAutoOp",
                label="Attributes", columns=2),
            VGroup("Parent", "ModelingAuthoritySet", "Measurements", "Hazards", "ErpOrganisationRoles", "DimensionsInfo", "ScheduledEvents", "Mediums", "AssetFunctions", "Properties", "AssetContainer", "Ratings", "ActivityRecords", "FromAssetRoles", "LocationRoles", "PowerSystemResourceRoles", "DocumentRoles", "ChangeItems", "ErpItemMaster", "ElectronicAddresses", "WorkTask", "ErpRecDeliveryItems", "ReliabilityInfos", "ToAssetRoles", "AssetPropertyCurves", "FinancialInfo", "ErpInventory", "acceptanceTest", "status", "Asset", "AssetFunctionAssetModel", "ComEquipmentAsset", "EndDeviceAsset", "Registers", "EndDeviceEvents", "Switches", "LoadMgmtRecords",
                label="References", columns=3),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfLoadControl.LoadMgmtFunction",
        title="LoadMgmtFunction",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "LoadMgmtFunction" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "LoadMgmtRecord" class:
#------------------------------------------------------------------------------

class LoadMgmtRecord(ActivityRecord):
    """ A log of actual measured load reductions as a result of load shed operations.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    LoadMgmtFunction = Instance("CIM.IEC61968.Informative.InfLoadControl.LoadMgmtFunction",
        transient=True,
        opposite="LoadMgmtRecords",
        editor=InstanceEditor(name="_loadmgmtfunctions"))

    def _get_loadmgmtfunctions(self):
        """ Property getter.
        """
        if self.Parent is not None:
            return [e for e in self.Parent.Elements \
                if "%s.%s" % (e.__module__, e.__class__.__name__) == \
                    "CIM.IEC61968.Informative.InfLoadControl.LoadMgmtFunction" ]
        else:
            return []

    _loadmgmtfunctions = Property(fget=_get_loadmgmtfunctions)

    # The measured reduction of the total load power as a result of the load shed activation. Thus it is the difference in power before and after the load shed operation.
    loadReduction = ActivePower(desc="The measured reduction of the total load power as a result of the load shed activation. Thus it is the difference in power before and after the load shed operation.")

    #--------------------------------------------------------------------------
    #  Begin "LoadMgmtRecord" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "reason", "category", "severity", "createdDateTime", "loadReduction",
                label="Attributes"),
            VGroup("Parent", "ModelingAuthoritySet", "MarketFactors", "Documents", "Organisations", "ScheduledEvent", "Assets", "ErpPersons", "Locations", "status", "LoadMgmtFunction",
                label="References", columns=1),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfLoadControl.LoadMgmtRecord",
        title="LoadMgmtRecord",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "LoadMgmtRecord" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "LoadLimitFunction" class:
#------------------------------------------------------------------------------

class LoadLimitFunction(LoadMgmtFunction):
    """ A kind of LoadMgmtFunction that limits the customer load to a given value.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # True if the switch will reconnect automatically, otherwise it will reconnect under manual control.
    isAutoReconOp = Bool(desc="True if the switch will reconnect automatically, otherwise it will reconnect under manual control.")

    # From the point when the maximumLoad threshold is crossed there may be a finite delay before the switch actually disconnects the load. Typically this is to buffer against transient load fluctuations.
    disconnectTimeDelay = Seconds(desc="From the point when the maximumLoad threshold is crossed there may be a finite delay before the switch actually disconnects the load. Typically this is to buffer against transient load fluctuations.")

    # From the point when the load recovers from an overload condition and crosses the maximumLoad threshold going down, there may be a finite time delay before the switch actually reconnects the load. Typically this is to give overload conditions sufficient time to clear, thus preventing unnecessary load switching activity.
    reconnectTimeDelay = Seconds(desc="From the point when the load recovers from an overload condition and crosses the maximumLoad threshold going down, there may be a finite time delay before the switch actually reconnects the load. Typically this is to give overload conditions sufficient time to clear, thus preventing unnecessary load switching activity.")

    # The power level, to which the customer load is being limited when this function activates. When the maximum load is exceeded the switch will typically open to shed the complete customer load.
    maximumLoad = ActivePower(desc="The power level, to which the customer load is being limited when this function activates. When the maximum load is exceeded the switch will typically open to shed the complete customer load.")

    #--------------------------------------------------------------------------
    #  Begin "LoadLimitFunction" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "initialCondition", "category", "lotNumber", "application", "serialNumber", "installationDate", "corporateCode", "purchasePrice", "manufacturedDate", "initialLossOfLife", "utcNumber", "critical", "hardwareID", "configID", "programID", "password", "firmwareID", "disabled", "schedulingBasis", "loadStatus", "manualOverRide", "overRideTimeOut", "remoteOverRide", "isAutoOp", "isAutoReconOp", "disconnectTimeDelay", "reconnectTimeDelay", "maximumLoad",
                label="Attributes", columns=2),
            VGroup("Parent", "ModelingAuthoritySet", "Measurements", "Hazards", "ErpOrganisationRoles", "DimensionsInfo", "ScheduledEvents", "Mediums", "AssetFunctions", "Properties", "AssetContainer", "Ratings", "ActivityRecords", "FromAssetRoles", "LocationRoles", "PowerSystemResourceRoles", "DocumentRoles", "ChangeItems", "ErpItemMaster", "ElectronicAddresses", "WorkTask", "ErpRecDeliveryItems", "ReliabilityInfos", "ToAssetRoles", "AssetPropertyCurves", "FinancialInfo", "ErpInventory", "acceptanceTest", "status", "Asset", "AssetFunctionAssetModel", "ComEquipmentAsset", "EndDeviceAsset", "Registers", "EndDeviceEvents", "Switches", "LoadMgmtRecords",
                label="References", columns=3),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfLoadControl.LoadLimitFunction",
        title="LoadLimitFunction",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "LoadLimitFunction" user definitions:
    #--------------------------------------------------------------------------

#------------------------------------------------------------------------------
#  "LoadShedFunction" class:
#------------------------------------------------------------------------------

class LoadShedFunction(LoadMgmtFunction):
    """ A kind of LoadMgmtFunction that sheds a part of the customer load.
    """

    #--------------------------------------------------------------------------
    #  Trait definitions:
    #--------------------------------------------------------------------------

    # The value of the load that is connected to the shedding switch. Typically this is a noted nominal value rather than a measured value.
    switchedLoad = ActivePower(desc="The value of the load that is connected to the shedding switch. Typically this is a noted nominal value rather than a measured value.")

    #--------------------------------------------------------------------------
    #  Begin "LoadShedFunction" user definitions:
    #--------------------------------------------------------------------------

    # @generated
    traits_view = View(Tabbed(
            VGroup("UUID", "description", "mRID", "name", "pathName", "localName", "aliasName", "initialCondition", "category", "lotNumber", "application", "serialNumber", "installationDate", "corporateCode", "purchasePrice", "manufacturedDate", "initialLossOfLife", "utcNumber", "critical", "hardwareID", "configID", "programID", "password", "firmwareID", "disabled", "schedulingBasis", "loadStatus", "manualOverRide", "overRideTimeOut", "remoteOverRide", "isAutoOp", "switchedLoad",
                label="Attributes", columns=2),
            VGroup("Parent", "ModelingAuthoritySet", "Measurements", "Hazards", "ErpOrganisationRoles", "DimensionsInfo", "ScheduledEvents", "Mediums", "AssetFunctions", "Properties", "AssetContainer", "Ratings", "ActivityRecords", "FromAssetRoles", "LocationRoles", "PowerSystemResourceRoles", "DocumentRoles", "ChangeItems", "ErpItemMaster", "ElectronicAddresses", "WorkTask", "ErpRecDeliveryItems", "ReliabilityInfos", "ToAssetRoles", "AssetPropertyCurves", "FinancialInfo", "ErpInventory", "acceptanceTest", "status", "Asset", "AssetFunctionAssetModel", "ComEquipmentAsset", "EndDeviceAsset", "Registers", "EndDeviceEvents", "Switches", "LoadMgmtRecords",
                label="References", columns=3),
            dock="tab"),
        id="CIM.IEC61968.Informative.InfLoadControl.LoadShedFunction",
        title="LoadShedFunction",
        buttons=["OK", "Cancel", "Help"],
        resizable=False)

    #--------------------------------------------------------------------------
    #  End "LoadShedFunction" user definitions:
    #--------------------------------------------------------------------------



# EOF -------------------------------------------------------------------------
